@@ -4,9 +4,7 @@ package Yahoo::Marketing::AdGroupResponse;
 
 use strict; use warnings;
 
-use Yahoo::Marketing::AdGroup;
-use Yahoo::Marketing::Error;
-use base qw/Yahoo::Marketing::ResponseType/;
+use base qw/Yahoo::Marketing::ComplexType/;
 
 =head1 NAME
 
@@ -19,6 +17,7 @@ sub _user_setable_attributes {
                  adGroup
                  errors
                  operationSucceeded
+                 warnings
             /  );
 }
 
@@ -31,16 +30,11 @@ __PACKAGE__->mk_accessors( __PACKAGE__->_user_setable_attributes,
                            __PACKAGE__->_read_only_attributes
                          );
 
-sub _type_field {
-    return 'adGroup';
-}
-
-
 
 1;
 =head1 SYNOPSIS
 
-See L<http://ysm.techportal.searchmarketing.yahoo.com/docs/reference/dataObjects.asp> for documentation of the various data objects.
+See L<http://searchmarketing.yahoo.com/developer/docs/V4/reference/dataObjects.php> for documentation of the various data objects.
 
 
 =cut
@@ -58,6 +52,7 @@ Creates a new instance
     adGroup
     errors
     operationSucceeded
+    warnings
 
 =back
 
