@@ -23,6 +23,7 @@ sub _read_only_attributes {
                  customizedResponseByAdGroup
                  defaultResponseByAdGroup
                  landscapeByAdGroup
+                 warnings
            / );
 }
 
@@ -34,7 +35,7 @@ __PACKAGE__->mk_accessors( __PACKAGE__->_user_setable_attributes,
 1;
 =head1 SYNOPSIS
 
-See L<http://ysm.techportal.searchmarketing.yahoo.com/docs/reference/dataObjects.asp> for documentation of the various data objects.
+See L<http://searchmarketing.yahoo.com/developer/docs/V3/reference/dataObjects.php> for documentation of the various data objects.
 
 
 =head2 new
@@ -52,6 +53,7 @@ Creates a new instance
     customizedResponseByAdGroup
     defaultResponseByAdGroup
     landscapeByAdGroup
+    warnings
 
 =back
 
