@@ -4,7 +4,6 @@ package Yahoo::Marketing::ForecastResponse;
 
 use strict; use warnings;
 
-use Yahoo::Marketing::ForecastResponseData;
 use base qw/Yahoo::Marketing::ComplexType/;
 
 =head1 NAME
@@ -15,13 +14,14 @@ Yahoo::Marketing::ForecastResponse - an object to represent a Yahoo Marketing Fo
 
 sub _user_setable_attributes {
     return ( qw/ 
+                 forecastLandscape
+                 forecastResponseDetail
+                 warnings
             /  );
 }
 
 sub _read_only_attributes {
     return ( qw/
-                 forecastLandscape
-                 forecastResponseDetail
            / );
 }
 
@@ -33,23 +33,24 @@ __PACKAGE__->mk_accessors( __PACKAGE__->_user_setable_attributes,
 1;
 =head1 SYNOPSIS
 
-See L<http://ysm.techportal.searchmarketing.yahoo.com/docs/reference/dataObjects.asp> for documentation of the various data objects.
+See L<http://searchmarketing.yahoo.com/developer/docs/V4/reference/dataObjects.php> for documentation of the various data objects.
 
 
-=head2 new
-
-Creates a new instance
-
 =cut
 
 =head1 METHODS
 
+=head2 new
+
+Creates a new instance
+
 =head2 get/set methods
 
 =over 8
 
     forecastLandscape
     forecastResponseDetail
+    warnings
 
 =back
 
