@@ -19,6 +19,7 @@ sub _user_setable_attributes {
                  market
                  maxKeywords
                  negativeKeywords
+                 offset
                  positiveKeywords
                  requiredPhraseFilters
             /  );
@@ -37,7 +38,7 @@ __PACKAGE__->mk_accessors( __PACKAGE__->_user_setable_attributes,
 1;
 =head1 SYNOPSIS
 
-See L<http://ysm.techportal.searchmarketing.yahoo.com/docs/reference/dataObjects.asp> for documentation of the various data objects.
+See L<http://searchmarketing.yahoo.com/developer/docs/V3/reference/dataObjects.php> for documentation of the various data objects.
 
 
 =head2 new
