@@ -6,6 +6,7 @@ use strict; use warnings;
 
 use base qw/ Test::Class Yahoo::Marketing::Test::PostTest /;
 use Test::More;
+use utf8;
 
 use Yahoo::Marketing::Keyword;
 use Yahoo::Marketing::KeywordService;
@@ -13,8 +14,6 @@ use Yahoo::Marketing::KeywordOptimizationGuidelines;
 
 # use SOAP::Lite +trace => [qw/ debug method fault /];
 
-my $section = 'sandbox';
-
 sub SKIP_CLASS {
     my $self = shift;
     # 'not running post tests' is a true value
@@ -50,7 +49,7 @@ sub test_can_copy_keyword : Test(4) {
     my $added_keyword = $self->create_keyword;
 
     # now copy the keyword to ad_group2
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     my $new_ss_bid = 99.99;
     ok( $ysm_ws->copyKeyword(
         keywordID => $added_keyword->ID,
@@ -86,7 +85,7 @@ sub test_can_move_keyword : Test(4) {
     my $added_keyword = $self->create_keyword;
 
     # now copy the keyword to ad_group2
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     my $new_ss_bid = 99.99;
     ok( $ysm_ws->moveKeyword(
         keywordID => $added_keyword->ID,
@@ -140,10 +139,42 @@ sub test_can_add_keyword : Test(8) {
     is(   $added_keyword->sponsoredSearchMaxBid, '1.0', 'sponsoredSearchMaxBid is set correctly' );
     like( $added_keyword->editorialStatus, qr/^Pending|Approved|Rejected|Suspended$/, 'editorialStatus seems right' );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     ok( $ysm_ws->deleteKeyword( keywordID => $added_keyword->ID ), 'delete new keyword' );
 }
 
+sub test_can_add_utf8_keyword : Test(10) {
+    my ( $self ) = @_;
+
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
+    my $word = 'ゲーム プレステ';
+
+    my $utf8_keyword = Yahoo::Marketing::Keyword->new
+                                                ->adGroupID( $self->common_test_data( 'test_ad_group' )->ID )
+                                                ->text( $word.$$ )
+                                                ->alternateText( "$word alternate text ".$$ )
+                                                ->sponsoredSearchMaxBid( 1 )
+                                                ->status( 'On' )
+                                                ->watchON( 'true' )
+                                                ->advancedMatchON( 'true' )
+                                                ->url( 'http://www.yahoo.com/utf8keyword?id='.$$ )
+                       ;
+
+    my $keyword = $ysm_ws->addKeyword( keyword => $utf8_keyword )->keyword;
+
+    ok(   $keyword, 'something was returned' );
+    like( $keyword->text, qr/$word\d+$/, 'text looks right' );
+    like( $keyword->alternateText, qr/$word alternate text \d+$/, 'alternate text looks right' );
+    is(   $keyword->url, 'http://www.yahoo.com/utf8keyword?id='.$$, 'url is right' );
+    like( $keyword->ID, qr/^[\d]+$/, 'ID is numeric' );
+    is(   $keyword->watchON, 'true', 'watchON is true' );
+    is(   $keyword->advancedMatchON, 'true', 'advancedMatchON is true' );
+    is(   $keyword->sponsoredSearchMaxBid, '1.0', 'sponsoredSearchMaxBid is set correctly' );
+    like( $keyword->editorialStatus, qr/^Pending|Approved|Rejected|Suspended$/, 'editorialStatus seems right' );
+
+    ok( $ysm_ws->deleteKeyword( keywordID => $keyword->ID ), 'delete new keyword' );
+}
+
 
 sub test_can_add_keywords : Test(22) {
     my ( $self ) = @_;
@@ -161,7 +192,7 @@ sub test_can_add_keywords : Test(22) {
         like( $added_keyword->editorialStatus, qr/^Pending|Approved|Rejected|Suspended$/, 'editorialStatus seems right' );
     }
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     ok( $ysm_ws->deleteKeywords( keywordIDs => [ map { $_->ID } @added_keywords ] ), 'delete new keywords' );
 }
 
@@ -173,7 +204,7 @@ sub test_can_delete_keyword : Test(3) {
 
     ok( $added_keyword, 'something was returned' );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     my $response = $ysm_ws->deleteKeyword( keywordID => $added_keyword->ID );
 
     is( $response->operationSucceeded, 'true' );
@@ -189,7 +220,7 @@ sub test_can_delete_keywords : Test(10) {
 
     ok( @added_keywords );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     my @responses = $ysm_ws->deleteKeywords( keywordIDs => [ map { $_->ID } @added_keywords ] );
 
     foreach my $response ( @responses ){
@@ -206,7 +237,7 @@ sub test_can_delete_keywords : Test(10) {
 sub test_can_get_keyword : Test(7) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my $fetched_keyword = $ysm_ws->getKeyword( keywordID => $self->common_test_data( 'test_keyword' )->ID );
 
@@ -222,7 +253,7 @@ sub test_can_get_keyword : Test(7) {
 sub test_can_get_keywords : Test(21) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my @fetched_keywords = $ysm_ws->getKeywords( keywordIDs => [ map { $_->ID } @{ $self->common_test_data( 'test_keywords' ) } ] );
 
@@ -241,7 +272,7 @@ sub test_can_get_keywords : Test(21) {
 sub test_can_get_keywords_by_account_id : Test(35) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my @fetched_keywords = $ysm_ws->getKeywordsByAccountID(
                                         accountID      => $ysm_ws->account,
@@ -266,7 +297,7 @@ sub test_can_get_keywords_by_account_id : Test(35) {
 sub test_can_get_keywords_by_ad_group_id : Test(35) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my @fetched_keywords = $ysm_ws->getKeywordsByAdGroupID(
                                         adGroupID      => $self->common_test_data( 'test_ad_group' )->ID,
@@ -292,7 +323,7 @@ sub test_can_get_keywords_by_ad_group_id : Test(35) {
 sub test_can_get_keywords_by_ad_group_id_by_editorial_status : Test(35) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my @fetched_keywords = $ysm_ws->getKeywordsByAdGroupIDByEditorialStatus(
                                         adGroupID       => $self->common_test_data( 'test_ad_group' )->ID,
@@ -323,7 +354,7 @@ sub test_can_get_keywords_by_ad_group_id_by_editorial_status : Test(35) {
 sub test_can_get_keywords_by_ad_group_id_by_status : Test(35) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my @fetched_keywords = $ysm_ws->getKeywordsByAdGroupIDByStatus(
                                         adGroupID       => $self->common_test_data( 'test_ad_group' )->ID,
@@ -347,7 +378,7 @@ sub test_can_get_keywords_by_ad_group_id_by_status : Test(35) {
 sub test_can_get_keyword_sponsored_search_max_bid : Test(1) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     is( $ysm_ws->getKeywordSponsoredSearchMaxBid( keywordID => $self->common_test_data( 'test_keyword' )->ID ), '1.0' );
 }
 
@@ -355,7 +386,7 @@ sub test_can_get_keyword_sponsored_search_max_bid : Test(1) {
 sub test_can_get_and_set_optimization_guidelines_for_keyword : Test(2) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my $keyword_optimization_guidelines = Yahoo::Marketing::KeywordOptimizationGuidelines->new
                                               ->accountID( $ysm_ws->account )
@@ -381,7 +412,7 @@ sub test_can_get_update_for_keyword : Test(3) {
 
     ok( $added_keyword, 'something was returned' );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     $ysm_ws->updateKeyword( keyword   => $added_keyword->alternateText('sex'),
                             updateAll => 'true',
@@ -401,7 +432,7 @@ sub test_can_get_editorial_reasons_for_keyword : Test(7) {
 
     ok( $added_keyword, 'something was returned' );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     my $new_keyword = $added_keyword->alternateText('drug oxycodone');
 
@@ -433,14 +464,14 @@ sub test_can_get_status_for_keyword : Test(2) {
 
     ok( $added_keyword, 'something was returned' );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     is( $ysm_ws->getStatusForKeyword( keywordID => $added_keyword->ID ), 'On' );
 }
 
 sub test_can_update_sponsored_search_max_bid_for_keyword : Test(2) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     $ysm_ws->updateSponsoredSearchMaxBidForKeyword(
         keywordID => $self->common_test_data( 'test_keyword' )->ID,
@@ -458,7 +489,7 @@ sub test_can_update_sponsored_search_max_bid_for_keyword : Test(2) {
 sub test_can_update_sponsored_search_max_bid_for_keywords : Test(6) {
     my ( $self ) = @_;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     $ysm_ws->updateSponsoredSearchMaxBidForKeywords(
         keywordIDs => [ map { $_->ID } @{$self->common_test_data( 'test_keywords' )} ],
@@ -485,7 +516,7 @@ sub test_can_update_keyword : Test(5) {
     my $old_alternate_text = $added_keyword->alternateText;
     my $old_url = $added_keyword->url;
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     my $response = $ysm_ws->updateKeyword( keyword   => $added_keyword->alternateText( 'apple' )
                                                                       ->url( 'http://www.somedomain.net' ),
                                            updateAll => 'true',
@@ -510,7 +541,7 @@ sub test_can_update_keywords : Test(7) {
         $keyword->url( 'http://www.somenewurl.net' );
     }
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
     $ysm_ws->updateKeywords( keywords  => \@added_keywords,
                              updateAll => 'true',
                            );
@@ -529,7 +560,7 @@ sub test_can_set_and_get_optimization_guidelines_for_keyword : Test(5) {
     my $keyword  = $self->common_test_data( 'test_keyword' );
     my $ad_group = $self->common_test_data( 'test_ad_group' );
 
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => 'sandbox' );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     $ysm_ws->setOptimizationGuidelinesForKeyword( 
                  optimizationGuidelines 
@@ -552,7 +583,7 @@ sub test_can_update_status_for_keyword : Test(2) {
     my ( $self ) = @_;
 
     my $added_keyword = $self->common_test_data( 'test_keyword' );
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     $ysm_ws->updateStatusForKeyword(
         keywordID => $added_keyword->ID,
@@ -572,7 +603,7 @@ sub test_can_update_status_for_keywords : Test(6) {
     my ( $self ) = @_;
 
     my @added_keywords = @{$self->common_test_data( 'test_keywords' )};
-    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $section );
+    my $ysm_ws = Yahoo::Marketing::KeywordService->new->parse_config( section => $self->section );
 
     $ysm_ws->updateStatusForKeywords(
         keywordIDs => [ map { $_->ID } @added_keywords ],
@@ -595,25 +626,10 @@ sub test_can_update_status_for_keywords : Test(6) {
 
 __END__
 
-* addKeyword
-* addKeywords
-* deleteKeyword
-* deleteKeywords
-* getKeyword
-* getKeywords
-* getKeywordsByAccountID
-* getKeywordsByAdGroupID
-* getKeywordsByAdGroupIDByEditorialStatus
-* getKeywordsByAdGroupIDByStatus
-* getKeywordSponsoredSearchMaxBid
-* getOptimizationGuidelinesForKeyword
-* getUpdateForKeyword
-* getEditorialReasonsForKeyword
-* getEditorialReasonText
-* getStatusForKeyword
-* setKeywordSponsoredSearchMaxBid
-* setOptimizationGuidelinesForKeyword
-* updateKeyword
-* updateKeywords
-* updateStatusForKeyword
-* updateStatusForKeywords
+* getKeywordsByAdGroupByParticipatesInMarketplace
+* getKeywordsByAdGroupBySponsoredSearchBidStatus
+* getReasonsForKeywordNotParticipatingInMarketplace
+* getSponsoredSearchMinBidForKeywordOptimizationGuidelines
+* getSponsoredSearchMinBidForKeywordString
+* getSponsoredSearchMinBidForKeywordStrings
+
